@@ -142,16 +142,20 @@ int main(int argc, char **argv)
 		if(FD_ISSET(s, &rfds))
 		{
 			if(bufFromD.sysread(s)<=0)
-				return 0;
+				goto finished;
 		}
 		
 		if(FD_ISSET(in, &rfds))
 		{
 			if(bufToD.sysread(in)<=0)
-				return 0;
-		}	
+				goto finished;
+		}
 	}
     
+	finished:
+	forceclose(s);
+	::shutdown(s, SHUT_RDWR);
+
 	return 0;
 }
 
