@@ -47,7 +47,7 @@ if($ENV{"ACNGIP"})
 elsif($cfg{bindaddress})
 {
    $cfg{remotehost}=$cfg{bindaddress};
-   $cfg{remotehost}=~s/\s*(\S+)\s*/$1/;
+   $cfg{remotehost}=~s/\s*(\S+)\s*.*/$1/; # only keep the first host
 }
 else
 {
