@@ -86,7 +86,7 @@ public:
 	acbuf databuf;
 	
 	
-	bool CheckAndPrecache(const char *path)
+	bool CheckOrExplore(const char *path)
 	{
 		static lockable mxCache;
 		static map<string, int> cache;
@@ -102,7 +102,7 @@ public:
 			}
 		}
 		// ok, not cached, do the hard way
-		bool res=Act(path, 0, 0, true);
+		bool res=Act(path, 0, 1, true);
 		_cerr("Act done, result: "<<res<<endl);
 		if(!res)
 			return res;
@@ -117,14 +117,14 @@ public:
 		return true;
 	}
 	
-	bool Act(const char *path, off_t pos, off_t len, bool bPrecache=false)
+	bool Act(const char *path, off_t pos, off_t len, bool bJustProbe=false)
 	{
 		_cerr( path << ", from: " << pos << " , " << len << "bytes\n");
 		databuf.clear();
 		h.clear();
 		
-		if(bPrecache)
-			len=150000; // fake something more sensible for now
+		if(bJustProbe)
+			len=100; // fake something more sensible for now
 		
 		if(!_Connect(false))
 			return false;
@@ -132,8 +132,8 @@ public:
 		string req(len==0?"HEAD ":"GET ");
 		
 		_cerr( "reqtype: " << req);
-		req+=baseUrl.ToString()+path+" HTTP/1.1\r\nConnection: Keep-Alive\r\nUser-Agent: ACNGFS\r\nX-Original-Source: 42\r\n";
-		if(!bPrecache && len>0)
+		req+=baseUrl.ToURI()+path+" HTTP/1.1\r\nConnection: Keep-Alive\r\nUser-Agent: ACNGFS\r\nX-Original-Source: 42\r\n";
+		if(!bJustProbe && len>0)
 		{
 			char buf[100];
 			sprintf(buf, "Range: bytes=%lu-%lu\r\n", (long unsigned)pos, (long unsigned) (pos+len-1));
@@ -183,7 +183,7 @@ public:
 				return true;
 			
 			// drop all stuff read during precaching
-			if(bPrecache && h.type!=h.INVALID)
+			if(bJustProbe && h.type!=h.INVALID)
 				databuf.clear();
 			
 			int nToRead=MIN(remaining,databuf.freecapa());
@@ -286,7 +286,7 @@ public:
 		
 		signal(SIGPIPE, SIG_IGN);
 		string sErr;
-		CAddrInfoPtr dns=_Resolve(proxyUrl.sHost, proxyUrl.sPort, sErr);
+    CAddrInfo::SPtr dns=CAddrInfo::CachedResolve(proxyUrl.sHost, proxyUrl.sPort, sErr);
 		if(!dns)
 		{
 			sErr="500 Cannot resolve remote address.";
@@ -333,7 +333,7 @@ bool GetFstat(const char *path, struct stat *stbuf, tDlDesc *d=NULL)
 		if (!d)
 			d=&e;
 
-		if (!d->CheckAndPrecache(path))
+		if (!d->CheckOrExplore(path))
 			return false;
 		if (d->seen_status!=200)
 			return false;
@@ -477,7 +477,7 @@ static int acngfs_open(const char *path, struct fuse_file_info *fi)
 		return -EIO;
 	}
 	
-	if(!p->CheckAndPrecache(path) ||  p->seen_status !=200)
+	if(!p->CheckOrExplore(path) ||  p->seen_status !=200)
 	{
 		delete p;
 		return -EISDIR;
