@@ -5,7 +5,7 @@
 #include "config.h"
 #include "meta.h"
 
-#define ACVERSION "0.3.4"
+#define ACVERSION "0.4.6"
 
 class hostdata;
 
@@ -16,7 +16,8 @@ extern MYSTD::string cachedir, logdir, confdir, fifopath, user, group, pidfile,
 reportpage, vfilepat, pfilepat, wfilepat, agentname, remoteport, adminauth, bindaddr, port, sUmask;
 
 extern int debug, offlinemode, foreground, verbose, stupidfs, forcemanaged, 
-verboselog, extreshhold, exfailabort, tpoolsize, dnscachetime, dlbufsize, exporigin, logxff;
+verboselog, extreshhold, exfailabort, tpstandbymax, tpthreadmax, dnscachetime, dlbufsize,
+exporigin, logxff, oldupdate, recompbz2, nettimeout, updinterval, forwardsoap;
 
 extern tHttpUrl proxy_info;
 
