@@ -7,8 +7,10 @@
 #include <map>
 #include <set>
 
-#include <netinet/in.h>
-#include <netdb.h>
+//#include <netinet/in.h>
+//#include <netdb.h>
+
+#include "tcpconnect.h"
 
 #include "lockable.h"
 #include "fileitem.h"
@@ -18,10 +20,10 @@
 class tDlJob;
 
 
-class dlcon : public lockable
+class dlcon : public lockable, tcpconnect
 { 
     public:
-        dlcon(bool bSingleRunMode, MYSTD::string *xff=NULL);
+        dlcon(bool bManualExecution, MYSTD::string *xff=NULL);
         ~dlcon();
                 
         void WorkLoop();
@@ -31,7 +33,7 @@ class dlcon : public lockable
 	    void AddJob(tFileItemPtr m_pItem, tHttpUrl hi);
         void AddJob(tFileItemPtr m_pItem, 
         		acfg::tHostiVec *pBackends, const MYSTD::string & sPatSuffix);
-        //void Reset();
+    	void EnqJob(tDlJob *);
 
         MYSTD::string m_sXForwardedFor;
         
@@ -39,12 +41,10 @@ class dlcon : public lockable
     	
     	friend class tDlJob;
     	
-    	void _AddJob(tDlJob *);
 
     	MYSTD::list<tDlJob*> m_qToReceive;
     	int m_wakepipe[2];
     	
-    	int m_conFd;
     	//MYSTD::string m_sConnectedHost; // used to remember to which host we are connected
 
     	acbuf m_InBuf;
@@ -54,15 +54,9 @@ class dlcon : public lockable
 
     	bool m_bSingleRun;
     	    
-    	tHttpUrl * m_proxy;
-    
     	//MYSTD::set<tHttpUrl*> m_MirrorBlacklist;
         MYSTD::set<MYSTD::string> m_MirrorHostBlacklist;
-        
-            	
-    	bool _Connect(const MYSTD::string & sHostname, MYSTD::string &sErrOut);
-    	inline void _Disconnect();
-    	
+
     	//not to be copied
     	dlcon & operator=(const dlcon&);
     	dlcon(const dlcon&);
