@@ -42,11 +42,21 @@ public:
 	// void Close();
 	bool CheckGoodState(bool bTerminateOnErrors);
 	
-	static bool GetMd5Sum(const MYSTD::string & sFileName, uint8_t out[], bool bTryUnpack, off_t &scannedSize);
-	static bool GetSha1Sum(const MYSTD::string & sFileName, uint8_t out[], bool bTryUnpack, off_t &scannedSize);
+	static bool GetMd5Sum(const MYSTD::string & sFileName, uint8_t out[], 
+			bool bTryUnpack, off_t &scannedSize);
+	
+	bool GetSha1Sum(uint8_t out[], off_t &scannedSize, const char *szUnpStoreFile=NULL);
+	
+	static bool GetSha1Sum(const MYSTD::string & sFileName, uint8_t out[], 
+			bool bTryUnpack, off_t &scannedSize,
+			const char *szUnpStoreFile=NULL);
+	
 	static bool GetMd5String(const MYSTD::string & sFileName, char out[]);
 	static bool CheckMd5Sum(const MYSTD::string & sFileName, const MYSTD::string & sReference);
-	char *GetBuffer(size_t & size) { size=m_nBufSize; return m_szFileBuf;};
+	inline const char *GetBuffer(size_t & size) { size=m_nBufSize; return m_szFileBuf;};
+	inline const char *GetBuffer() { return m_szFileBuf; };
+	inline size_t GetSize() { return m_nBufSize; };
+	void Close();
 	
 private:
 	
@@ -78,5 +88,20 @@ private:
 
 };
 
+namespace Meta
+{
+extern uint_fast16_t hexmap[];
+
+inline bool CsEqual(CUCHAR a[], uint8_t b[], unsigned short binLen)
+{
+	for(int i=0; i<binLen;i++)
+	{
+		uint_fast16_t r=hexmap[a[i*2]] * 16 + hexmap[a[i*2+1]];
+		if(r != b[i]) return false;
+	}
+	return true;
+};
+}
+
 #endif // __FILEREADER_H
 
