@@ -16,12 +16,26 @@ class rechecks {
     	} eFileKind;
     	
         rechecks();
-        ~rechecks();
+        /* virtual, not needed yet */ ~rechecks();
         //bool hasPoison(const char *taint);
         eFileKind getFiletype(const char *tstr);
-
-    private:
+    
+    protected:
         regex_t /*rePoison, */reIfiles, rePfiles;
+        int m_nErrors;
+
+};
+
+class rechecksEx : public rechecks 
+{
+public:
+	rechecksEx();
+	~rechecksEx();
+	bool IsInWhitelist(const char*);
+	inline bool IsRegexOk() { return m_nErrors; };
+
+protected:
+	regex_t reWfiles;
 };
 
 #endif // _rechecks_h_
