@@ -6,9 +6,11 @@
 class reportgen : public maintenance
 {
 public:
-	reportgen(int);
+	reportgen(int fd, bool bVerbose=false);
 	virtual void Run(const MYSTD::string &);
-	virtual void GenReport();
+protected:
+	virtual void GenReport(const MYSTD::string & cmd);
+	bool m_bVerbose;
 };
 
 #endif /*REPORTGEN_H_*/
