@@ -57,11 +57,11 @@ bool acbuf::initFromFile(const char *szPath)
 	{
 		if (sysread(fd) < 0)
 		{
-			close(fd);
+			forceclose(fd);
 			return false;
 		}
 	}
-	close(fd);
+	forceclose(fd);
 	return true;
 }
 
