@@ -38,37 +38,44 @@ char alphabet[65] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456
 string sPopularPath("/debian/");
 
 MapNameToString n2sTbl[] = {
-		{  "Port", 			&port }
-		,{  "CacheDir", 	&cachedir }
-		,{ "LogDir", 	&logdir }
-		,{ "SocketPath", 	&fifopath}
-		,{ "PidFile", 	&pidfile}
-		,{ "Proxy",		&proxy}
-		,{ "ReportPage",&reportpage}
-		,{ "VfilePattern", &vfilepat}
-		,{ "PfilePattern", &pfilepat}
-		,{ "WfilePattern", &wfilepat}
-		,{ "AdminAuth",  &adminauth}
-		,{ "BindAddress", &bindaddr}
-		,{ "UserAgent", &agentname}
+		{  "Port", 			&port , 0}
+		,{  "CacheDir", 	&cachedir, 0 }
+		,{ "LogDir", 	&logdir , 0}
+		,{ "SocketPath", 	&fifopath, 0}
+		,{ "PidFile", 	&pidfile, 0}
+		,{ "Proxy",		&proxy, 0}
+		,{ "ReportPage",&reportpage, 0}
+		,{ "VfilePattern", &vfilepat, 0}
+		,{ "PfilePattern", &pfilepat, 0}
+		,{ "WfilePattern", &wfilepat, 0}
+		,{ "AdminAuth",  &adminauth, 0}
+		,{ "BindAddress", &bindaddr, 0}
+		,{ "UserAgent", &agentname, 0}
 		//,{ "Umask",		  &sUmask}
 };
 
 MapNameToInt n2iTbl[] = {
-		{ "Debug", 		&debug }
-		,{ "OfflineMode", 	&offlinemode }
-		,{ "ForeGround", 	&foreground }
-		//,{ "Verbose", 		&verbose }
-		,{ "ForceManaged", 	&forcemanaged }
-		,{ "StupidFs", 		&stupidfs }
-		,{ "VerboseLog",	&verboselog }
-		,{ "ExTreshold",	&extreshhold} 
-		,{ "MaxSpareThreadSets",	&tpoolsize}
-		,{ "DnsCacheSeconds", &dnscachetime}
-		,{ "UnbufferLogs", &debug }
-		,{ "ExAbortOnProblems", &exfailabort}
-		,{ "ExposeOrigin", &exporigin}
-		,{ "LogSubmittedOrigin", &logxff}
+		{ "Debug", 		&debug, 0 }
+		,{ "OfflineMode", 	&offlinemode , 0}
+		,{ "ForeGround", 	&foreground , 0}
+		,{ "Verbose", 		0, "Option is deprecated, ignoring the value." }
+		,{ "ForceManaged", 	&forcemanaged , 0}
+		,{ "StupidFs", 		&stupidfs , 0}
+		,{ "VerboseLog",	&verboselog , 0}
+		,{ "ExTreshold",	&extreshhold, 0}
+		,{ "MaxSpareThreadSets",	&tpstandbymax, "Deprecated option name, mapped to MaxStandbyConThreads"}
+		,{ "MaxStandbyConThreads",	&tpstandbymax, 0}
+		,{ "MaxConThreads",	&tpthreadmax, 0}
+		,{ "DnsCacheSeconds", &dnscachetime, 0}
+		,{ "UnbufferLogs", &debug , 0}
+		,{ "ExAbortOnProblems", &exfailabort, 0}
+		,{ "ExposeOrigin", &exporigin, 0}
+		,{ "LogSubmittedOrigin", &logxff, 0}
+		,{ "OldIndexUpdater", &oldupdate, "Option is deprecated, ignoring the value." }
+		,{ "RecompBz2", &recompbz2, 0}
+		,{ "NetworkTimeout", &nettimeout, 0}
+		,{ "MinUpdateInterval", &updinterval, 0}
+		,{ "ForwardBtsSoap", &forwardsoap, 0}
 };
 
 #define _iterPos(it, start) (it-start.begin())/sizeof(it)
@@ -99,7 +106,12 @@ tMapString2Hostivec mapRepName2Backends;
 string * _GetStringPtr(const string &key)  {
 	for(unsigned int i=0; i<_countof(n2sTbl); i++) {
 		if(0==strcasecmp(key.c_str(), n2sTbl[i].name))
+		{
+			if(n2sTbl[i].warn)
+				cerr << "Warning, " << key << ": " << n2sTbl[i].warn << endl; 
+						
 			return n2sTbl[i].ptr;
+		}
 	}
 	return NULL;
 }
@@ -107,7 +119,12 @@ string * _GetStringPtr(const string &key)  {
 int * _GetIntPtr(const string &key)  {
 	for(unsigned int i=0; i<_countof(n2iTbl); i++) {
 		if(0==strcasecmp(key.c_str(), n2iTbl[i].name))
+		{
+			if(n2iTbl[i].warn)
+				cerr << "Warning, " << key << ": " << n2iTbl[i].warn << endl;
+			
 			return n2iTbl[i].ptr;
+		}
 	}
 	return NULL;
 }
@@ -182,6 +199,8 @@ inline void _AddRemapInfo(bool bAsBackend, const string & token,
 		
 		if (sPath[0]!=cPathSep)
 			sPath.insert(0, confdir+sPathSep);
+		
+		tStrVec srcs;
 
 #ifdef HAVE_WORDEXP
 		wordexp_t p;
@@ -189,32 +208,28 @@ inline void _AddRemapInfo(bool bAsBackend, const string & token,
 		if(0==wordexp(sPath.c_str(), &p, 0))
 		{
 			for (UINT i=0; i<p.we_wordc; i++)
-			{
-				const string sToRead = p.we_wordv[i];
+				srcs.push_back(p.we_wordv[i]);
+			wordfree(&p);
+		}
+		
 #elif defined HAVE_GLOB
 		glob_t globbuf;
 		memset(&globbuf, 0, sizeof(glob_t));
 		if(0==glob(sPath.c_str(), GLOB_DOOFFS | GLOB_NOSORT, NULL, &globbuf))
 		{
-			for (unsigned int i=0; i<globbuf.gl_pathc; i++)
-			{
-				const string sToRead = globbuf.gl_pathv[i];
-#else
-				const string & sToRead = sPath;
-#endif		
-				if (bAsBackend)
-					_ReadBackendsFiles(sToRead, repname);
-				else
-					_ReadRewriteFiles(sToRead, repname);			
-#ifdef HAVE_WORDEXP
-			}
-			wordfree(&p);
-		}
-#elif defined HAVE_GLOB	
-			}
+			for (UINT i=0; i<globbuf.gl_pathc; i++)
+				srcs.push_back(globbuf.gl_pathv[i]);
 			globfree(&globbuf);
 		}
 #endif
+		std::sort(srcs.begin(), srcs.end());
+		for(tStrVecIterConst it=srcs.begin(); it!=srcs.end(); it++)
+		{
+			if (bAsBackend)
+				_ReadBackendsFiles(*it, repname);
+			else
+				_ReadRewriteFiles(*it, repname);
+		}
 	}
 }
 
@@ -250,8 +265,17 @@ bool SetOption(const MYSTD::string &sLine, bool bQuiet)
 		}
 		
 		errno=0;
-		char *pEnd;
-		*pnTarget=strtol(pStart, &pEnd, 10);
+		char *pEnd(0);
+		long nVal = strtol(pStart, &pEnd, 10);
+
+		if(RESERVED_DEFVAL == nVal)
+		{
+			cerr << "Bad value for " << key << " (protected value, use another one)" <<endl;
+			return false;
+		}
+
+		*pnTarget=nVal;
+
 		if (errno)
 		{
 			cerr << "Invalid number for " << key << " ";
@@ -367,13 +391,21 @@ void _ReadBackendsFiles(const string & sFile, const string &sRepName)
 		//	cerr << "backends, got line: " << sLine <<endl;
 		
 
-		if(0==sLine.compare(0, 7, "http://") && entry.SetHttpUrl(sLine))
+		if( (startsWithSz(sLine, "http://") && entry.SetHttpUrl(sLine) )
+				||
+			(IsValidButIrrelevantLine(sLine) 
+					&& ! entry.sHost.empty() 
+					&& ! entry.sPath.empty()
+			)
+		)
 		{
 			_FixPostPreSlashes(entry.sPath);
-			mapRepName2Backends[sRepName].push_back(entry);
 #ifdef DEBUG
-			cerr << "Backend: " << sRepName << " <-- " << entry.ToString() <<endl;
-#endif			
+			cerr << "Backend: " << sRepName << " <-- " << entry.ToURI() <<endl;
+#endif		
+
+			mapRepName2Backends[sRepName].push_back(entry);
+			entry.clear();
 		}
 		else if(_ParseLine(sLine, key, val))
 		{
@@ -393,20 +425,12 @@ void _ReadBackendsFiles(const string & sFile, const string &sRepName)
 				}
 			}
 			*/
-			else if(keyEq("Archive-http", key) || keyEq("X-Archive-http", key) 
-					|| keyEq("Volatile-http",key) || keyEq("X-Volatile-http", key))
+			else if(keyEq("Archive-http", key) || keyEq("X-Archive-http", key))
 			{
-				_FixPostPreSlashes(val);
 				entry.sPath=val;
 			}
-		
 		}
-		else if(IsValidButIrrelevantLine(sLine))  
-		{
-			mapRepName2Backends[sRepName].push_back(entry);
-
-		}
-		else
+		else if(bNotEof && !IsValidButIrrelevantLine(sLine))
 		{
 			cerr << "Bad backend description, around line "
 					<< reader.GetPositionDescription() << endl;
@@ -415,6 +439,12 @@ void _ReadBackendsFiles(const string & sFile, const string &sRepName)
 	}
 }
 
+void ShutDown()
+{
+	mapUrl2pVname.clear();
+	mapRepName2Backends.clear();
+}
+
 void _ReadRewriteFiles(const string & sFile, const string & sRepName)
 {
 
@@ -455,7 +485,10 @@ void _ReadRewriteFiles(const string & sFile, const string & sRepName)
 			if (hosts.empty() && paths.empty())
 				continue; // dummy run
 			if ( !hosts.empty() && paths.empty())
-				paths.push_back("/");
+			{
+				cerr << "Warning, missing path spec for the site " << hosts[0] <<", ignoring mirror."<< endl;
+				continue;
+			}
 			if ( !paths.empty() && hosts.empty())
 			{
 				cout << "Parse error, missing Site: field around line "
@@ -497,8 +530,7 @@ void _ReadRewriteFiles(const string & sFile, const string & sRepName)
 		if( keyEq("Site", key) || keyEq("Alias", key) || keyEq("Aliases", key))
 			Tokenize(val, SPACECHARS, hosts, true);
 		
-		if(keyEq("Archive-http", key) || keyEq("X-Archive-http", key) 
-				|| keyEq("Volatile-http",key) || keyEq("X-Volatile-http", key))
+		if(keyEq("Archive-http", key) || keyEq("X-Archive-http", key))
 		{
 			// help STL saving some memory
 			if(sPopularPath==val)
@@ -588,7 +620,7 @@ void PostProcConfig()
 	
     // postprocessing
 
-#if 0
+#ifdef FORCE_CUSTOM_UMASK
 	if(!sUmask.empty())
 	{
 		mode_t nUmask=0;
@@ -625,6 +657,9 @@ void PostProcConfig()
 	   exit(EXIT_FAILURE);
    }
    
+   if(acfg::tpthreadmax < 0)
+	   acfg::tpthreadmax = numeric_limits<int>::max();
+	   
    // get rid of duplicated and trailing slash(es)
    tStrVec tmp;
    Tokenize(cachedir, sPathSep.c_str(), tmp);
@@ -683,11 +718,22 @@ void PostProcConfig()
    if(! pidfile.empty())
 	   mkbasedir(acfg::pidfile);
 
+   if(nettimeout < 5) {
+	   cerr << "Warning, NetworkTimeout too small, assuming 5." << endl;
+	   nettimeout = 5;
+   }
+
+   if(RESERVED_DEFVAL == forwardsoap)
+	   forwardsoap = !forcemanaged;
+
 #ifdef DEBUG
    if (acfg::debug>8)
 	{
 		for (UINT i=0; i<_countof(n2sTbl); i++)
 		{
+			if(! n2sTbl[i].ptr)
+				continue;
+
 			string tmp, *val=n2sTbl[i].ptr;
 			cout << n2sTbl[i].name << " = " << *val <<endl;
 			// escaped version
@@ -696,7 +742,8 @@ void PostProcConfig()
 			cout << n2sTbl[i].name << " = " << tmp << endl;
 		}
 		for (UINT i=0; i<_countof(n2iTbl); i++)
-			cout << n2iTbl[i].name << " = \"" << *(n2iTbl[i].ptr) << "\"\n";
+			if(n2iTbl[i].ptr)
+				cout << n2iTbl[i].name << " = \"" << *(n2iTbl[i].ptr) << "\"\n";
 	}
 #else
    if(acfg::debug>2)
