@@ -114,9 +114,9 @@ void reopen()
 {
 	lockguard g(&mx);
 	if (fErr)
-		fclose(fErr);
+		forceFclose(fErr);
 	if (fStat)
-		fclose(fStat);
+		forceFclose(fStat);
 	aclog::open();
 }
 
@@ -126,24 +126,47 @@ void close()
 	if(acfg::debug>4) cerr << "Closing logs...\n";
 	if(fErr) 
 	{
-		fclose(fErr);
+		forceFclose(fErr);
 		fErr=NULL;
 	}
 	if(fStat) 
 	{
-		fclose(fStat);
+		forceFclose(fStat);
 		fStat=NULL;
 	}
 }
 
+
+#define DAYSECONDS (3600*24)
+#define WEEKSECONDS (DAYSECONDS * 7)
+
 void GetStats(vector<tRowData> & out)
 {
 	string sDataFile=acfg::cachedir+sPathSep+"_stats_dat";
+
+	
+	out.clear();
+	time_t now=time(NULL);
+	
+	for(int i=0; i<7; i++)
+	{
+		tRowData d;
+		d.to=now - i*DAYSECONDS;
+		d.from=d.to - DAYSECONDS;
+		out.push_back(d);
+	}
+	
+#warning NOTE: stats cache disabled, restore it later?
+#if 0
 	struct stat statbuf;
 	
 	if ( 0!= stat(sDataFile.c_str(), &statbuf) 
-			|| statbuf.st_mtime < time(NULL)-86000)
+			|| statbuf.st_mtime < time(NULL)-86000
+	   )
+
 	{ // needs to be created/updated
+		
+#endif
 		glob_t globbuf;
 
 		memset(&globbuf, 0, sizeof(glob_t));
@@ -152,8 +175,6 @@ void GetStats(vector<tRowData> & out)
 
 		//cout << "wo? " << (acfg::logdir+"/*.log.*").c_str() <<endl;
 		
-		map<time_t,pair<uint64_t,uint64_t> > mapPeriod2InOut;
-		time_t now=time(NULL);
 		for (unsigned int i=0; i<globbuf.gl_pathc; i++)
 		{
 			if(acfg::debug)
@@ -166,6 +187,7 @@ void GetStats(vector<tRowData> & out)
 			}
 			string sLine;
 			tStrVec tokens;
+
 			while(reader.GetOneLine(sLine))
 			{
 				// cout << "got line: " << sLine <<endl;
@@ -176,34 +198,41 @@ void GetStats(vector<tRowData> & out)
 				 // cout << "having: " << tokens[0] << ", " << tokens[1] << ", " << tokens[2]<<endl;
 				 
 				 time_t when=strtoul(tokens[0].c_str(),0,10);
-
-				 time_t delta=now-when;
-				 if(when>now || delta>=3600*24*28)
+				 if(when > out.front().to || when < out.back().from)
 					 continue;
 				 
-				 // uber-maximum of days we would ever display, use as offset to mark rows for weekly style presentation 
-#define RANGEMARK 2048
-				 
-				 time_t key; // map to particular weeks or DOW
-				 if(delta>=7*24*3600) // account for weeks
-					key = RANGEMARK + delta / (7*3600*24); 
-				 else // account for days
-					 key = delta / (24*3600);
-				 
-				 unsigned long count=strtoul(tokens[2].c_str(),0,10);
-				 
-				 switch(* tokens[1].c_str()) {
-				 case('I'): mapPeriod2InOut[key].first+=count; break;
-				 case('O'): mapPeriod2InOut[key].second+=count; break;
-				 default: continue;
+				 for(vector<tRowData>::reverse_iterator it=out.rbegin();
+				 it!=out.rend(); 
+				 it++)
+				 {
+					  if(when < it->from || when > it->to)
+						  continue;
+
+						 unsigned long dcount=strtoul(tokens[2].c_str(),0,10);
+						 switch(* tokens[1].c_str()) {
+						 case('I'): 
+							 it->byteIn+=dcount;
+						 it->reqIn++;
+						 break;
+						 case('O'): 
+							 it->byteOut+=dcount;
+						 it->reqOut++;
+						 break;
+						 default:
+							 continue;
+						 }
+	  
 				 }
+				 
+				 
 			}
 		}
 		globfree(&globbuf);
 		
-		// Converting stats to (almost) user readable output for caller 
+#if 0
+		// Normalize data and repack for user 
 		
-		map<time_t,pair<uint64_t,uint64_t> >::iterator it;
+		map<time_t,tempCounts>::iterator it;
 		for(it=mapPeriod2InOut.begin(); it!=mapPeriod2InOut.end(); it++)
 		{
 			tRowData data;
@@ -219,7 +248,6 @@ void GetStats(vector<tRowData> & out)
 			out.push_back(data);
 			
 		}
-
 	}
 	else
 	{ // deserialize
@@ -243,6 +271,7 @@ void GetStats(vector<tRowData> & out)
 			tokens.clear();
 		}
 	}
+#endif
 }
 	
 errnoFmter::errnoFmter()
@@ -252,7 +281,7 @@ errnoFmter::errnoFmter()
 	msg=&buf[0];
 
 #ifdef _GNU_SOURCE
-#warning COMPILER BUG DETECTED -- _GNU_SOURCE is set in C++ mode
+#warning COMPILER BUG DETECTED -- _GNU_SOURCE was preset in C++ mode
 	msg=strerror_r(errno, buf, sizeof(buf)-1);
 #else
 	strerror_r(errno, buf, sizeof(buf)-1);
