@@ -7,7 +7,7 @@
 
 using namespace MYSTD;
 
-static map<string, CAddrInfoPtr> mapDnsCache;
+static map<string, CAddrInfo::SPtr> mapDnsCache;
 static pthread_mutex_t lockDnsCache= PTHREAD_MUTEX_INITIALIZER;
 
 CAddrInfo::CAddrInfo() :
@@ -42,8 +42,9 @@ bool CAddrInfo::Resolve(const string & sHostname, const string &sPort,
 
 	if (0!=r)
 	{
-		sErrorBuf="503 DNS error: ";
-		sErrorBuf+=gai_strerror(r);
+		sErrorBuf=string("503 DNS error for hostname ")+sHostname+": "
+				+gai_strerror(r)+". If "+sHostname
+				+" refers to a configured cache repository, please check the corresponding configuration file.";
 		return false;
 	}
 
@@ -71,12 +72,12 @@ CAddrInfo::~CAddrInfo()
 }
 	
 
-CAddrInfoPtr _Resolve(const string & sHostname, const string &sPort, string &sErrorMsgBuf)
+CAddrInfo::SPtr CAddrInfo::CachedResolve(const string & sHostname, const string &sPort, string &sErrorMsgBuf)
 {
-	CAddrInfoPtr p;
+	SPtr p;
 	time_t timeExpired=time(NULL)-acfg::dnscachetime;
 
-	map<string, CAddrInfoPtr>::iterator it;
+	map<string, SPtr>::iterator it;
 	{
 		lockguard g(&lockDnsCache);
 		it=mapDnsCache.find(sHostname);
