@@ -8,6 +8,7 @@
 #include "header.h"
 #include "dlcon.h"
 #include "acbuf.h"
+#include "tcpconnect.h"
 
 #include <sys/select.h>
 #include <signal.h>
@@ -55,30 +56,19 @@ void con::SignalStop() {
 }
 #endif
 
-void con::ShutDown()
-{
-	// no double calls
-	if(m_confd>=0) 
-	{
-		shutdown(m_confd, SHUT_RDWR);
-		close(m_confd);
-		m_confd=-1;
-	}
-}
 
 //void DumpItems();
 
 con::~con() {
 	ldbg("Destroying connection...");
-    
-#ifdef KILLABLE
-	if (wakepipe[0]>=0)
-		close(wakepipe[0]);
 
-	if (wakepipe[1]>=0)
-		close(wakepipe[1]);
-#endif
-	
+	if(m_confd>=0) 
+	{
+		shutdown(m_confd, SHUT_RDWR);
+		forceclose(m_confd);
+		m_confd=-1;
+	}
+
 	MYSTD::list<job*>::iterator jit;
 	for (jit=m_jobs2send.begin(); jit!=m_jobs2send.end(); jit++)
 		delete *jit;
@@ -103,6 +93,97 @@ con::~con() {
     aclog::flush();
 }
 
+class SoapPassThrough : public tcpconnect
+{
+public:
+	bool TrySoapPassThrough(acbuf &clientBufIn, int fdClient)
+	{
+
+#define POSTMARK "POST http://bugs.debian.org:80/"
+#define FAKEMARK "POST                          /"
+
+		if(clientBufIn.size()<_countof(POSTMARK)-1)
+			return false;
+		if(strncmp(clientBufIn.rptr(), POSTMARK, _countof(POSTMARK)-1))
+			return false;
+
+		string sErr;
+		if(!_Connect("bugs.debian.org", sErr))
+			return false;
+
+
+		acbuf clientBufOut; // use custom buffers?
+		clientBufOut.init(32*1024); // out to be enough for any BTS response
+
+		// for convenience
+		int fdServer = m_conFd;
+		acbuf &serverBufOut = clientBufIn, &serverBufIn = clientBufOut;
+
+		int maxfd=1+std::max(fdClient, fdServer);
+
+		while (true)
+		{
+			fd_set rfds, wfds;
+			FD_ZERO(&rfds);
+			FD_ZERO(&wfds);
+
+			// can send to client?
+			if(clientBufOut.size()>0)
+				FD_SET(fdClient, &wfds);
+
+			// can receive from client?
+			if(clientBufIn.freecapa()>0)
+				FD_SET(fdClient, &rfds);
+
+			if(serverBufOut.size()>0)
+				FD_SET(fdServer, &wfds);
+
+			if(serverBufIn.freecapa()>0)
+				FD_SET(fdServer, &rfds);
+
+			int nReady=select(maxfd, &rfds, &wfds, NULL, NULL);
+			if (nReady<0)
+				return false;
+
+			if(FD_ISSET(fdServer, &wfds))
+			{
+				// if there is some other proxy, let it deal with the stuff
+				if(!m_proxy)
+				{
+					serverBufOut.move();
+					char *p = 0;
+					while (0 != (p = strstr(serverBufOut.rptr(), POSTMARK)))
+						memcpy(p, FAKEMARK, _countof(FAKEMARK) - 1);
+				}
+
+				if(serverBufOut.syswrite(fdServer)<0)
+					return false;
+			}
+
+			if(FD_ISSET(fdClient, &wfds))
+			{
+				if(clientBufOut.syswrite(fdClient)<0)
+					return false;
+			}
+
+			if(FD_ISSET(fdServer, &rfds))
+			{
+				if(serverBufIn.sysread(fdServer)<=0)
+					return true;
+			}
+
+			if(FD_ISSET(fdClient, &rfds))
+			{
+				if(clientBufIn.sysread(fdClient)<=0)
+					return true;
+			}
+		}
+
+		return false;
+
+	};
+};
+
 void con::WorkLoop() {
     
 	signal(SIGPIPE, SIG_IGN);
@@ -110,12 +191,7 @@ void con::WorkLoop() {
     acbuf inBuf;
     inBuf.init(32*1024);
     
-#ifdef KILLABLE
-    int maxfd=MYSTD::max(wakepipe[0], m_confd);
-#else
     int maxfd=m_confd;
-#endif
-    
     while(!m_bStopActivity) {
         fd_set rfds, wfds;
         FD_ZERO(&wfds);
@@ -126,13 +202,7 @@ void con::WorkLoop() {
         	return; // shouldn't even get here
         
         job *pjSender(NULL);
-        
-        
-#ifdef KILLABLE
-        // prepare to send data or to be notified by helper
-        FD_SET(wakepipe[0], &rfds);
-#endif
-
+    
         if ( !m_jobs2send.empty())
 		{
 			pjSender=m_jobs2send.front();
@@ -185,7 +255,7 @@ void con::WorkLoop() {
 					return; // no resources? whatever
 				
 				int nConsumed=m_pTmpHead->LoadFromBuf(inBuf.rptr(), inBuf.size());
-				//ldbg("header parsed how? " << nConsumed);
+				ldbg("header parsed how? " << nConsumed);
 				if(nConsumed==0)
 				{ // Either not enough data received, or buffer full; make space and retry
 					inBuf.move();
@@ -193,7 +263,15 @@ void con::WorkLoop() {
 				}
 				if(nConsumed<0)
 				{
-					ldbg("Bad request");
+					 // also must be identified before
+					if( acfg::forwardsoap && ! m_sClientHost.empty() )
+					{
+						SoapPassThrough pt;
+						if(pt.TrySoapPassThrough(inBuf, m_confd))
+							return;
+					}
+
+					ldbg("Bad request: " << inBuf.rptr() );
 					return;
 				}
 				if (nConsumed>0)
@@ -239,7 +317,7 @@ void con::WorkLoop() {
 		if(FD_ISSET(m_confd, &wfds) && pjSender)
 		{
 			ldbg("Sending data of " << pjSender);
-			switch(pjSender->SendData(m_confd))
+			switch(pjSender->SendData(m_confd, m_jobs2send.size()))
 			{
 				case(job::R_DISCON):
 				{
