@@ -26,7 +26,7 @@
 
 using namespace MYSTD;
 
-// for cygwin, incomplete ipv6 stuff
+// for cygwin, incomplete ipv6 support
 #ifndef AF_INET6
 #define AF_INET6        23              /* IP version 6 */
 #endif
@@ -34,83 +34,85 @@ using namespace MYSTD;
 namespace conserver
 {
 
-int sockunix(-1);
-vector<int> vecSocks;
+int g_sockunix(-1);
+vector<int> g_vecSocks;
 
-condition cond;
-
-list<con*> /* qForHelper, */ qForWorker;
-//map<con*,int> mConStatus;
-//int nStandbyHelper(0);
-//int nStandbyWorker(0);
-int nSpareThreads(0);
+condition g_ThreadPoolCondition;
+list<con*> g_freshConQueue;
+int g_nStandbyThreads(0);
+int g_nAllConThreadCount(0);
 bool bTerminationMode(false);
-int nAllConThreadCount(0);
 
-//#define TYPE_WORKER ((void*)0)
-//#define TYPE_HELPER ((void*)1)
-#define STANDBY_LIMIT acfg::tpoolsize
+// safety mechanism, detect when the number of incomming connections 
+// is growing A LOT faster than it can be processed 
 #define MAX_BACKLOG 200
 
 void * ThreadAction(void *)
 {
-	list<con*> & myq = qForWorker;
+	list<con*> & Qu = g_freshConQueue;
 	USRDBG(3, "Thread started");
 
-	lockguard g(cond);
-	nAllConThreadCount++;
-
+	lockguard g(g_ThreadPoolCondition);
+	
 	while (true)
 	{
 		if (bTerminationMode)
 			break;
 
-		if(myq.empty())
-		{ // to be decreased by the pool client
-			nSpareThreads++;
-			while (myq.empty())
-				cond.wait();
+		if(Qu.empty())
+		{
+			g_nStandbyThreads++;
+			while (Qu.empty())
+				g_ThreadPoolCondition.wait();
+			g_nStandbyThreads--;
 		}
-				
-		con *c=myq.front();
-		myq.pop_front();
+
+		con *c=Qu.front();
+		Qu.pop_front();
 		g.unLock();
 	
 		c->WorkLoop();
-		// should be dead and unreferenced now
-#ifdef KILLABLE
-		// should be already free, but better be safe...
-		c->SignalStop(); 
-#endif
-		c->ShutDown(); // close file descriptors
 		delete c;
 		
 		g.reLock();
-		if (nSpareThreads>=STANDBY_LIMIT)
-			return NULL;
+		if (g_nStandbyThreads >= acfg::tpstandbymax)
+			goto stop_worker;
 	}
 	
-	nAllConThreadCount--;
+	stop_worker:
+	g_nAllConThreadCount--;
 	USRDBG(3, "Terminating thread");
 	return NULL;
 }
 
-bool SpawnThread(void * type)
+// Thread creation helper, global stuff must be protected while using this
+bool SpawnThread()
 {
-	USRDBG(3, "Creating " << (type? "helper" : "worker"));
+	USRDBG(3, "Creating connection worker");
+	
+	 // artificial limit hit?
+	if(g_nAllConThreadCount>=acfg::tpthreadmax)
+		return false;
+	
+	g_nAllConThreadCount++;
+	
 	pthread_t thr;
-	pthread_attr_t attr; // detached, to be initialized in Setup
+	pthread_attr_t attr; // be detached from the beginning 
 	pthread_attr_init(&attr);
 	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
 
-	return (0==pthread_create(&thr, NULL, ThreadAction, type));
+	if(0==pthread_create(&thr, NULL, ThreadAction, NULL))
+		return true;
+	
+	g_nAllConThreadCount--;
+	return false;
 }
 
 void SetupConAndGo(int fd, const char *szClientName=NULL)
 {
-	// protect from interference of con shutdown with OS on file descriptor 
-	// management and also thread pool control
-	lockguard g(cond);
+	// thread pool control, and also see Shutdown(), protect from 
+	// interference of OS on file descriptor management
+	lockguard g(g_ThreadPoolCondition);
 	
 	if(!szClientName)
 		szClientName="";
@@ -136,19 +138,17 @@ void SetupConAndGo(int fd, const char *szClientName=NULL)
 	}
 	
 	// DOS prevention
-	if (qForWorker.size() > MAX_BACKLOG)
+	if (g_freshConQueue.size() > MAX_BACKLOG)
 	{
 		USRDBG(4, "Worker queue overrun");
 		goto failure_mode;
 	}
 
-	if (nSpareThreads>0)
-		nSpareThreads--; // one is ours
-	else if(!SpawnThread(0))
-		goto failure_mode;
+	if (g_nStandbyThreads==0 && !SpawnThread())
+	    goto failure_mode;
 	
-	qForWorker.push_back(c);
-	cond.notifyAll();
+	g_freshConQueue.push_back(c);
+	g_ThreadPoolCondition.notifyAll();
 
 	return;
 
@@ -157,7 +157,7 @@ void SetupConAndGo(int fd, const char *szClientName=NULL)
 		delete c;
 	shutdown(fd, SHUT_RDWR);
 	USRDBG(3, "Connection setup error");
-	close(fd);
+	forceclose(fd);
 }
 
 void CreateUnixSocket() {
@@ -180,18 +180,18 @@ void CreateUnixSocket() {
 	mkbasedir(sPath);
 	unlink(sPath.c_str());
 	
-	sockunix = socket(PF_UNIX, SOCK_STREAM, 0);
-	setsockopt(sockunix, SOL_SOCKET, SO_REUSEADDR, &jo, sizeof(jo));
+	g_sockunix = socket(PF_UNIX, SOCK_STREAM, 0);
+	setsockopt(g_sockunix, SOL_SOCKET, SO_REUSEADDR, &jo, sizeof(jo));
 
-	if (sockunix<0)
+	if (g_sockunix<0)
 		goto unx_err;
 	
-	if (bind(sockunix, (struct sockaddr *)&addr_unx, size) < 0)
+	if (bind(g_sockunix, (struct sockaddr *)&addr_unx, size) < 0)
 		goto unx_err;
 	
-	if (0==listen(sockunix, SO_MAXCONN))
+	if (0==listen(g_sockunix, SO_MAXCONN))
 	{
-		vecSocks.push_back(sockunix);
+		g_vecSocks.push_back(g_sockunix);
 		return;
 	}
 	
@@ -225,7 +225,7 @@ void Setup()
 		
 		tStrVec sAdds;
 		if(bindaddr.empty())
-			sAdds.push_back(""); // dummy, use any later
+			sAdds.push_back(""); // one dummy entry to get one NULL later
 		else
 			Tokenize(bindaddr, SPACECHARS, sAdds);
 		for(tStrVec::iterator it=sAdds.begin(); it!=sAdds.end(); it++)
@@ -243,7 +243,11 @@ void Setup()
 				if (sockip<0)
 					goto error_socket;
 
-
+          // if we have a dual-stack IP implementation (like on Linux) then
+          // explicitely disable the shadow v4 listener. Otherwise it might be
+          // bound, or maybe not, and then maybe because of the dual-behaviour,
+          // or maybe because of real errors; we just cannot know for sure but
+          // we have to.
 #ifdef IPV6_V6ONLY
 		    	if(p->ai_family==AF_INET6)
 		    		setsockopt(sockip, SOL_IPV6, IPV6_V6ONLY, &yes, sizeof(yes));
@@ -256,7 +260,7 @@ void Setup()
 		    	if (listen(sockip, SO_MAXCONN))
 		    		goto error_listen;
 		    	
-		    	vecSocks.push_back(sockip);
+		    	g_vecSocks.push_back(sockip);
 		    	
 		    	continue;
 
@@ -277,10 +281,14 @@ void Setup()
 
 				error_bind:
 				perror("Couldn't bind socket");
+				cerr.flush();
+				if(EADDRINUSE == errno)
+					cerr << "Port " << port << " is busy, see the manual (Troubleshooting chapter) for details." <<endl;
+				cerr.flush();
 				goto close_socket;
 
 				close_socket:
-				close(sockip);
+				forceclose(sockip);
 				sockip=-1;
 		    }
 		    freeaddrinfo(res);
@@ -289,7 +297,7 @@ void Setup()
 		    error_getaddrinfo:
 		    perror("Error resolving address for binding");
 		}
-		if(vecSocks.empty())
+		if(g_vecSocks.empty())
 		{
 			cerr << "No socket(s) could be create/prepared. "
 			"Check the network, check or unset the BindAddress directive.\n";
@@ -309,7 +317,7 @@ int Run()
 {
 	fd_set rfds, wfds;
 	int maxfd(0);
-	for(vector<int>::iterator it=vecSocks.begin(); it!=vecSocks.end(); it++)
+	for(vector<int>::iterator it=g_vecSocks.begin(); it!=g_vecSocks.end(); it++)
 		if(*it > maxfd)
 			maxfd=*it;
 	maxfd++;
@@ -320,7 +328,7 @@ int Run()
 
 		FD_ZERO(&rfds);
 		FD_ZERO(&wfds);
-		for(vector<int>::iterator it=vecSocks.begin(); it!=vecSocks.end(); it++)
+		for(vector<int>::iterator it=g_vecSocks.begin(); it!=g_vecSocks.end(); it++)
 			FD_SET(*it, &rfds);
 		
 		//cerr << "Polling..." <<endl;
@@ -335,19 +343,24 @@ int Run()
 			exit(EXIT_FAILURE);
 		}
 		
-		for(vector<int>::iterator it=vecSocks.begin(); it!=vecSocks.end(); it++)
+		for(vector<int>::iterator it=g_vecSocks.begin(); it!=g_vecSocks.end(); it++)
 		{
-			if(!FD_ISSET(*it, &rfds))
-			continue;
-			if(sockunix == *it)
+			if(!FD_ISSET(*it, &rfds)) 	continue;
+
+			if(g_sockunix == *it)
 			{
-				int fd = accept(sockunix, NULL, NULL);
+				int fd = accept(g_sockunix, NULL, NULL);
 				if (fd>=0)
 				{
 					set_nb(fd);
 					USRDBG(1, "Detected incoming connection from the UNIX socket");
 					SetupConAndGo(fd);
 				}
+				else if(errno == EMFILE || errno == ENOMEM || ENOBUFS == errno)
+				{
+					// play nicely, give it a break
+					sleep(1);
+				}
 			}
 			else
 			{
@@ -355,7 +368,7 @@ int Run()
 
 				socklen_t addrlen = sizeof(addr);
 				int fd=accept(*it,(struct sockaddr *)&addr, &addrlen);
-
+//fd_accepted:
 				if (fd>=0)
 				{
 					set_nb(fd);
@@ -364,11 +377,19 @@ int Run()
 					if (getnameinfo((struct sockaddr*) &addr, addrlen, hbuf, sizeof(hbuf),
 									NULL, 0, NI_NUMERICHOST))
 					{
-						printf("ERROR: could not resolve hostname\n");
-						return 1;
+						aclog::err("ERROR: could not resolve hostname for incoming TCP host");
+						::shutdown(fd, SHUT_RDWR);
+						forceclose(fd);
+						sleep(1);
+						continue;
 					}
 					SetupConAndGo(fd, hbuf);
 				}
+				else if(errno == EMFILE || errno == ENOMEM || ENOBUFS == errno)
+				{
+					// play nicely, give it a break
+					sleep(1);
+				}
 			}
 		}
 	}
@@ -377,7 +398,7 @@ int Run()
 
 void Shutdown()
 {
-	lockguard g(cond);
+	lockguard g(g_ThreadPoolCondition);
 	
 	if(bTerminationMode)
 		return; // double SIGWHATEVER? Prevent it.
@@ -389,15 +410,15 @@ void Shutdown()
 	
 	bTerminationMode=true;
 	printf("Notifying waiting threads\n");
-	cond.notifyAll();
+	g_ThreadPoolCondition.notifyAll();
 	
 	printf("Closing sockets\n");
-	for(vector<int>::iterator it=vecSocks.begin(); it!=vecSocks.end(); it++)
+	for(vector<int>::iterator it=g_vecSocks.begin(); it!=g_vecSocks.end(); it++)
 	{
 		if(*it>0)
 		{
 			shutdown(*it, SHUT_RDWR);
-			close(*it);
+			forceclose(*it);
 			*it=-1;
 		}
 	}
