@@ -8,7 +8,7 @@
 #include <vector>
 //#include <iostream>
 
-#include "config.h"
+#include "meta.h"
 #include "dirwalk.h"
 
 using namespace MYSTD;
@@ -16,7 +16,7 @@ using namespace MYSTD;
 struct dnode
 {
 	dnode(dnode *parent) : m_parent(parent) {};
-	void Walk(IFileHandler *);
+	bool Walk(IFileHandler *);
 
 	MYSTD::string sPath;
 	dnode *m_parent;
@@ -29,20 +29,16 @@ private:
 
 };
 
-void dnode::Walk(IFileHandler *h)
+bool dnode::Walk(IFileHandler *h)
 {
 	if (stat(sPath.c_str(), &m_stinfo) < 0)
-		return; // slight risk of missing information here... bug ignoring is safer
+		return true; // slight risk of missing information here... bug ignoring is safer
 	
 	if(S_ISREG(m_stinfo.st_mode))
-	{
-		h->ProcessRegular(sPath, m_stinfo);
-		return;
-	}
+		return h->ProcessRegular(sPath, m_stinfo);
 	else if(! S_ISDIR(m_stinfo.st_mode))
 	{
-		h->ProcessOthers(sPath, m_stinfo);
-		return;
+		return h->ProcessOthers(sPath, m_stinfo);
 	}
 	
 	// ok, that's a directory, scan it and descend where needed
@@ -50,36 +46,43 @@ void dnode::Walk(IFileHandler *h)
 	for(dnode *cur=m_parent; cur!=NULL; cur=cur->m_parent)
 	{
 		if (m_stinfo.st_dev == cur->m_stinfo.st_dev && m_stinfo.st_ino == cur->m_stinfo.st_ino)
-			return;
+			return true;
 	}
 	
 //	cerr << "Opening: " << sPath<<endl;
 	DIR *dir = opendir(sPath.c_str());
 	if (!dir) // weird, whatever... ignore...
-		return;
+		return true;
 	
 	struct dirent *dp;
 	dnode childbuf(this);
+	bool bRet(true);
 	
 	while ( NULL != (dp = readdir(dir)) )
 	{
 		if (strcmp(dp->d_name, ".") && strcmp(dp->d_name, ".."))
 		{
 			childbuf.sPath=sPath+string(SZPATHSEP)+dp->d_name;
-			childbuf.Walk(h);
+			bRet=childbuf.Walk(h);
+			if(!bRet)
+				goto stop_walk;
 		}
 	}
 	
+	
 //	cerr << "Closing: " << sPath<<endl;
-	closedir(dir);
+	stop_walk:
+	if(dir)
+		closedir(dir);
+	return bRet;
 }
 
 
 
-void DirectoryWalk(string & sRoot, IFileHandler *h)
+bool DirectoryWalk(string & sRoot, IFileHandler *h)
 {
 	dnode root(NULL);
-	root.sPath=sRoot;
-	root.Walk(h);
+	root.sPath=sRoot; 
+	return root.Walk(h);
 }
 
