@@ -7,8 +7,7 @@
 #include "dlcon.h"
 
 #include "fileitem.h"
-#include "caddrinfo.h"
-#include "sockio.h"
+#include <errno.h>
 #include "dljob.h"
 
 #define MAX_RETRIES 3
@@ -16,9 +15,9 @@
 using namespace MYSTD;
 
 
-dlcon::dlcon(bool bSingleRunMode, string *xff):
-	m_conFd(-1),
-m_bSingleRun(bSingleRunMode)
+
+dlcon::dlcon(bool bManualExecution, string *xff) :
+m_bSingleRun(bManualExecution)
 {
 	ldbg("Creating dlcon");
 	if (0==pipe(m_wakepipe))
@@ -29,12 +28,11 @@ m_bSingleRun(bSingleRunMode)
 	else
 		m_wakepipe[0]=m_wakepipe[1]=-1;
 	
-	m_proxy = acfg::proxy_info.sHost.empty() ? NULL : &acfg::proxy_info;
 	if(xff)
 		m_sXForwardedFor=*xff;
 }
 
-inline void dlcon::_AddJob(tDlJob *todo)
+void dlcon::EnqJob(tDlJob *todo)
 {
 	setLockGuard;
 	
@@ -47,12 +45,12 @@ void dlcon::AddJob(tFileItemPtr m_pItem,
 		acfg::tHostiVec *pBackends, const MYSTD::string & sPatSuffix)
 {
 	//lockguard g(m_pItem.get());
-	_AddJob(new tDlJob(this, m_pItem, pBackends, sPatSuffix));
+	EnqJob(new tDlJob(this, m_pItem, pBackends, sPatSuffix));
 }
 void dlcon::AddJob(tFileItemPtr m_pItem, tHttpUrl hi)
 {
 	//lockguard g(m_pItem.get());
-	_AddJob(new tDlJob(this, m_pItem, hi.sHost, hi.sPath));
+	EnqJob(new tDlJob(this, m_pItem, hi.sHost, hi.sPath));
 }
 
 void dlcon::SignalStop()
@@ -75,13 +73,12 @@ void dlcon::SignalStop()
 dlcon::~dlcon()
 {
 	ldbg("Destroying dlcon");
-	_Disconnect();
 	
 	if (m_wakepipe[0]>=0)
-		close(m_wakepipe[0]);
+		forceclose(m_wakepipe[0]);
 
 	if (m_wakepipe[1]>=0)
-		close(m_wakepipe[1]);
+		forceclose(m_wakepipe[1]);
 		
 	ldbg("destroying dlcon done. Any postponed tasks to do? ");
 }
@@ -170,7 +167,7 @@ void dlcon::WorkLoop()
 					continue;
 				}
 				
-				const string sTargetHost = pj->GetHost();
+				const string sTargetHost = pj->GetPeerName();
 				__lockguard.unLock();
 				// might override it with proxy stuff
 				bool bOk=_Connect(sTargetHost, sErrorMsg);
@@ -233,7 +230,7 @@ void dlcon::WorkLoop()
 				for(it=m_qToReceive.begin(); it!=m_qToReceive.end(); it++)
 				{
 					dbgline;
-					if(! (*it)->MakeRequest(sTargetHost, sSendBuf))
+					if(! (*it)->AppendRequest(sTargetHost, sSendBuf))
 					{
 						// refused -> needs to reconnect later
 						(*it)->bSuggestReconnect=true;
@@ -269,7 +266,7 @@ void dlcon::WorkLoop()
 		do_select:
 		
 		ldbg("select dlcon");
-		tv.tv_sec = 60;
+		tv.tv_sec = acfg::nettimeout;
 		tv.tv_usec = 0;
 		
 		r=select(nMaxFd+1, &rfds, &wfds, NULL, &tv);
@@ -339,7 +336,7 @@ void dlcon::WorkLoop()
 					// found the first unseen, do stuff
 					
 					pjn->bRequestPrepared=true;
-					if(!pjn->FindConfig() || !pjn->MakeRequest(sCurrentlyConnectedHost, sSendBuf)) 
+					if(!pjn->FindConfig() || !pjn->AppendRequest(sCurrentlyConnectedHost, sSendBuf)) 
 					{
 						// bad stuff, check later
 						bStopRequesting=true;
@@ -419,7 +416,7 @@ void dlcon::WorkLoop()
 
 		while( ! m_InBuf.empty() && pj)
 		{
-			ldbg("Processing input of " << pj->ToString() );
+			ldbg("Processing input of " << pj->RemoteUri() );
 			tDlJob::tDlResult res=pj->ProcessIncomming(m_InBuf, sErrorMsg);
 			ldbg("... input processing result: " << res);
 			switch(res)
@@ -431,6 +428,7 @@ void dlcon::WorkLoop()
 				// item was ours but download aborted... might do more fine recovery in the future
 				case(tDlJob::R_ERROR_LOCAL):
 				case(tDlJob::R_ERROR_REMOTE):
+				case(tDlJob::R_NEXTSTATE): // this one should never appear, but who knows...
 					goto drop_and_restart_stream;
 									
 				case(tDlJob::R_DONE):
@@ -465,65 +463,3 @@ void dlcon::WorkLoop()
 	}
 }
 
-bool dlcon::_Connect(const string & sHostname, string & sErrorMsg)
-{
-	_Disconnect();
-	ldbg("Resolving " + sHostname);
-	CAddrInfoPtr dns = m_proxy ? 
-			_Resolve(m_proxy->sHost, m_proxy->sPort, sErrorMsg) 
-			: _Resolve(sHostname, acfg::remoteport, sErrorMsg);
-	if(!dns)
-	{
-		return false; // sErrorMsg got the info already, no other change to undo
-		ldbg("DNS error");
-	}
-		
-
-	::signal(SIGPIPE, SIG_IGN);
-	
-	
-	for (struct addrinfo *pInfo=dns->m_bestInfo; pInfo; pInfo = pInfo->ai_next)
-	{
-		ldbg("Creating socket for " << sHostname);		
-
-		if (pInfo->ai_socktype != SOCK_STREAM || pInfo->ai_protocol != IPPROTO_TCP)
-			continue;
-		
-		m_conFd = ::socket(pInfo->ai_family, pInfo->ai_socktype, pInfo->ai_protocol);
-		if(m_conFd<0)
-			continue;
-
-#ifndef NO_TCP_TUNNING
-		{
-			int yes(1);
-			::setsockopt(m_conFd,SOL_SOCKET,SO_REUSEADDR,&yes,sizeof(yes));
-			::setsockopt(m_conFd, IPPROTO_TCP, TCP_NODELAY, &yes, sizeof(yes));
-		}
-#endif
-
-		if(::connect(m_conFd, pInfo->ai_addr, pInfo->ai_addrlen) < 0)
-			continue;
-		
-		ldbg("connect() ok");
-		set_nb(m_conFd);
-		return true;
-	}
-	
-	// format the last available error message for the user
-	aclog::errnoFmter fer;		    
-	sErrorMsg="500 Connection failure: ";
-	sErrorMsg+=fer.msg;
-	ldbg("Force reconnect, con. failure");
-	_Disconnect();
-	return false;
-}
-
-inline void dlcon::_Disconnect()
-{
-	dbgline;
-	if(m_conFd<0)
-		return;
-	::shutdown(m_conFd, O_RDWR);
-	::close(m_conFd);
-	m_conFd=-1;	
-}
