@@ -3,6 +3,8 @@
 #define LOCAL_DEBUG
 #include "debug.h"
 
+//#define _XOPEN_SOURCE 600 // for posix_fallocate from fcntl.h
+
 #include "config.h"
 #include "fileitem.h"
 #include "header.h"
@@ -19,6 +21,12 @@
 #define O_BINARY 0 // ignore on Unix
 #endif
 
+#ifdef HAVE_LINUX_FALLOCATE
+int falloc_helper(int fd, off_t start, off_t len);
+#else
+#define falloc_helper(fd,start,len)
+#endif
+
 using namespace MYSTD;
 
 header const * fileitem::GetHeaderUnlocked()
@@ -38,7 +46,43 @@ fileitem::fileitem(MYSTD::string sPath) :
 	m_nDlRefsCount(0),
 	status(FIST_FRESH)
 {
-   ldbg("item created, m_sPath: "<< m_sPath);
+
+	// not using hexmap, just converting only few allowed chars
+
+	for (int i = 0; i < int(m_sPath.length()) - 3; i++)
+	{
+		if (m_sPath[i] == '%')
+		{
+			switch (m_sPath[i + 1])
+			{
+			case '7':
+				switch (m_sPath[i + 2])
+				{
+				case 'E':
+				case 'e':
+					m_sPath.replace(i, 3, "~", 1);
+					break;
+				}
+				break;
+			case '5':
+				switch (m_sPath[i + 2])
+				{
+				case 'B':
+				case 'b':
+					m_sPath.replace(i, 3, "[", 1);
+					break;
+				case 'D':
+				case 'd':
+					m_sPath.replace(i, 3, "]", 1);
+					break;
+				}
+				break;
+			default:
+				continue;
+			}
+		}
+	}
+	ldbg("item created, m_sPath: "<< m_sPath);
 }
 
 void fileitem::AddDownloaderRef()
@@ -69,7 +113,7 @@ void fileitem::DelDownloaderRef(const string &sReason)
 	}
 	if (m_filefd>=0)
 	{
-::		close(m_filefd);
+		forceclose(m_filefd);
 		m_filefd=-1;
 	}
 }
@@ -121,6 +165,7 @@ FiStatus fileitem::Setup(bool bCheckFreshness, bool bIgnoreCached) {
 		if (0==::stat(m_sPath.c_str(), &stbuf))
 			m_nSizeSeen=stbuf.st_size;
 		
+
 		// some plausibility checks
 		if(m_bCheckFreshness)
 		{
@@ -129,7 +174,7 @@ FiStatus fileitem::Setup(bool bCheckFreshness, bool bIgnoreCached) {
 				goto error_clean; // suspicious, cannot use it
 		}
 		else
-		{ 
+		{
 			// non-volatile files, so could accept the length, do some checks first
 			const char *pContLen=m_head.h[header::CONTENT_LENGTH];
 			if(pContLen)
@@ -153,7 +198,7 @@ FiStatus fileitem::Setup(bool bCheckFreshness, bool bIgnoreCached) {
 			}				
 		}
 	}
-	else
+	else // -> no .head file
 	{
 		if(bCheckFreshness)
 		{
@@ -175,18 +220,6 @@ FiStatus fileitem::Setup(bool bCheckFreshness, bool bIgnoreCached) {
 
 inline void _LogWithErrno(const char *msg, const string & sFile)
 {
-	/*
-	char buf[32];
-	buf[0]=buf[31]=0x0;
-	const char *szErr=&buf[0];
-
-#ifdef _GNU_SOURCE
-#warning COMPILER BUG DETECTED -- _GNU_SOURCE is set in C++ mode
-	szErr=strerror_r(errno, buf, sizeof(buf)-1);
-#else
-	strerror_r(errno, buf, sizeof(buf)-1);
-#endif
-	*/
 	aclog::errnoFmter f;
 	aclog::err( (sFile+" storage error ["+msg+"]: "+f.msg).c_str());
 }
@@ -204,6 +237,9 @@ bool fileitem::DownloadStartedStoreHeader(const header & h)
 		status=FIST_DLGOTHEAD; // assume that for now, may become error state...
 	
 	m_nIncommingCount+=h.m_nEstimLength;
+
+	// optional optimization: hints for the filesystem resp. kernel
+	off_t hint_start(0), hint_length(0);
 	
 	// status will change, most likely... ie. BOUNCE action
 	notifyAll();
@@ -222,6 +258,8 @@ bool fileitem::DownloadStartedStoreHeader(const header & h)
 		bStoreData=true;
 		m_nSizeChecked=0;
 		m_head=h;
+		const char *p=h.h[header::CONTENT_LENGTH];
+		hint_length=(off_t) (p ? atoll(p) : 0);
 	}
 	else if(206 == code)
 	{
@@ -254,10 +292,14 @@ bool fileitem::DownloadStartedStoreHeader(const header & h)
 		m_nSizeChecked=myfrom;
 		bStoreData=true;
 		
+		hint_start=(off_t) myfrom;
+		hint_length=(off_t) mylen;
+
 		m_head=h;
 		m_head.frontLine="HTTP/1.1 200 OK";
 		m_head.del(header::CONTENT_RANGE);
 		m_head.set(header::CONTENT_LENGTH, mylen);
+		m_head.set(header::XORIG, h.h[header::XORIG]);
 	}
 	else if(416 == code) // that's always bad; it cannot be complete (-1 trick) -> kill cached file ASAP
 	{
@@ -269,7 +311,11 @@ bool fileitem::DownloadStartedStoreHeader(const header & h)
 		// have a clear one with just the error message
 		m_head.frontLine=h.frontLine;
 		m_head.type=header::ANSWER;
-		
+		m_head.set(header::XORIG, h.h[header::XORIG]);
+
+		// pass-through the new location if needed
+		if(code == 301 || code == 302)
+			m_head.set(header::LOCATION, h.h[header::LOCATION]);
 	}
 			
 	if(bStoreData)
@@ -333,9 +379,11 @@ bool fileitem::DownloadStartedStoreHeader(const header & h)
 		}
 		else if(m_nSizeChecked>m_nSizeSeen) // should never happen and caught by the checks above
 			SETERRORKILLFILE("503 Internal error on size checking");
-		// else... nothing to change if the expectation==reality
+		// else... nothing to fix since the expectation==reality
+
+		falloc_helper(m_filefd, hint_start, hint_length);
 		
-		ldbg("Opening "+sHeadPath);
+		ldbg("Storing header as "+sHeadPath);
 		int count=m_head.StoreToFile(sHeadPath);
 		
 		// unlink and retry
