@@ -14,7 +14,7 @@ using namespace MYSTD;
 #include "dlcon.h"
 #include "maintenance.h"
 
-#ifdef HAVE_SENDFILE
+#ifdef HAVE_LINUX_SENDFILE
 #include <sys/sendfile.h>
 #endif
 
@@ -75,31 +75,26 @@ ssize_t sendfile_generic(int out_fd, int in_fd, off_t *offset, size_t count)
 	return cnt;
 }
 
-#ifdef HAVE_SENDFILE
-ssize_t linux_sendfile_with_fallback(int out_fd, int in_fd, off_t *offset, size_t count)
+ssize_t sendfile_linuxstyle(int out_fd, int in_fd, off_t *offset, size_t count)
 {
+#ifndef HAVE_LINUX_SENDFILE
+	return sendfile_generic(out_fd, in_fd, offset, count);
+#else
 	ssize_t r=sendfile(out_fd, in_fd, offset, count);
-	
+
 	if(r<0 && (errno==ENOSYS || errno==EINVAL))
 		return sendfile_generic(out_fd, in_fd, offset, count);
 	else
 		return r;
+#endif
+	
 }
-#else
-#define linux_sendfile_with_fallback(out_fd, in_fd, poffset, count) sendfile_generic(out_fd, in_fd, poffset, count)
-#endif
-
-#ifndef __linux__
-#define sendfile_linux(a,b,c,d) sendfile_generic(a,b,c,d)
-#else
-#define sendfile_linux(a,b,c,d) linux_sendfile_with_fallback(a,b,c,d)
-#endif
 
 job::job(header *h, con *pParent) :
 	m_filefd(-1),
 	m_pParentCon(pParent),
 	m_bChunkMode(false),
-	m_bCloseCon(false),
+	//m_bCloseCon(false),
 	m_bIsHttp11(false),
 	m_state(STATE_FRESH),
 	m_pReqHead(h),
@@ -120,7 +115,7 @@ job::~job()
 			m_nAllDataCount);
 	
 	if (m_filefd>=0)
-		close(m_filefd);
+		forceclose(m_filefd);
 
 	if (m_pItem)
 	{
@@ -174,12 +169,14 @@ void job::PrepareDownload() {
     
 
 	{
+		/*
 		// persistent connection?
 	    const char *p=m_pReqHead->h[header::CONNECTION];
-		if (!p) // default: "close" IFF not http 1.1
-			m_bCloseCon = !m_bIsHttp11; 
+		if (!p) 
+			m_bCloseCon = true; // enough! too many stupid clients out there; previous default: "close" IFF not http 1.1 !m_bIsHttp11; 
 		else // normalize case
 			m_bCloseCon = (0 == strcasecmp(p, "close"));
+		*/
 		
 		tStrVec tmp;
 		if(3 == Tokenize(m_pReqHead->frontLine, SPACECHARS, tmp))
@@ -191,7 +188,8 @@ void job::PrepareDownload() {
     MYTRY
 	{
     	// filesystem browsing attempt?
-		if(stmiss != sTmp.find("..")) goto report_notallowed;
+		if(stmiss != sTmp.find("..") || stmiss != sTmp.find("/_actmp"))
+			goto report_notallowed;
 
 		if (0==sTmp.compare(0, 12, "apt-cacher?/"))
 		sTmp.erase(0, 12);
@@ -210,7 +208,7 @@ void job::PrepareDownload() {
 		for(tStrPos pos; stmiss != (pos = sPath.find("//")); )
 			sPath.erase(pos, 1);
 
-		ldbg("input uri: "<<tUrl.ToString());
+		ldbg("input uri: "<<tUrl.ToURI());
 
 		tStrPos nRepNameLen=acfg::reportpage.size();
 		if(nRepNameLen>0 && 0==tUrl.sHost.compare(0, nRepNameLen, acfg::reportpage))
@@ -253,6 +251,17 @@ void job::PrepareDownload() {
     }
     
     bForceFreshnessChecks = ( ! acfg::offlinemode && m_type==rechecks::FILE_INDEX);
+
+#ifdef QUATSCH
+    // some users want to not be most up-to-date
+	if(bForceFreshnessChecks && acfg::updinterval > 0)
+	{
+		struct stat stbuf;
+		bForceFreshnessChecks=(0 == ::stat(m_sFileLoc.c_str(), &stbuf)
+				&& ( time(0) - (UINT) stbuf.st_ctime) < acfg::updinterval);
+	}
+#endif
+
     m_pItem=fileitem::GetFileItem(m_sFileLoc);
     
     if(!m_pItem)
@@ -298,7 +307,7 @@ void job::PrepareDownload() {
 			    if(acfg::forcemanaged)
 			    	goto report_notallowed;
 			    
-			    ldbg("Passing new job for " << tUrl.ToString() << " to " << m_pParentCon->m_dlerthr);
+			    ldbg("Passing new job for " << tUrl.ToURI() << " to " << m_pParentCon->m_dlerthr);
 			    m_pParentCon->m_pDlClient->AddJob(m_pItem, tUrl);
 			}
 			ldbg("Download job enqueued for " << m_sFileLoc);
@@ -339,7 +348,7 @@ report_invport:
 }
 
 #define THROW_ERROR(x) { m_sErrorMsgBuffer=x; goto THROWN_ERROR; }
-job::eJobResult job::SendData(int confd)
+job::eJobResult job::SendData(int confd, int nAllJobCount)
 {
 	if(!m_sMaintCmd.empty())
 	{
@@ -349,7 +358,7 @@ job::eJobResult job::SendData(int confd)
 	
 	ldbg("job::SendData");
 	
-	off_t nNewSize(0);
+	off_t nGoodSize(0);
 	FiStatus fistate(FIST_ERROR);
 
 	if (confd<0|| m_state==STATE_FAILMODE)
@@ -361,7 +370,7 @@ job::eJobResult job::SendData(int confd)
 	}
 	else if(m_nCachedMaxSize>0)
 	{  // short circuit the fitem check -- sendfile was interrupted, just continue there 
-		nNewSize=m_nCachedMaxSize;
+		nGoodSize=m_nCachedMaxSize;
 	}
 	else if (m_pItem)
 	{
@@ -370,7 +379,7 @@ job::eJobResult job::SendData(int confd)
 		while (true)
 		{
 			fistate=m_pItem->GetStatusUnlocked();
-			nNewSize=m_pItem->m_nSizeChecked;
+			nGoodSize=m_pItem->m_nSizeChecked;
 			
 			dbgline;
 			if (fistate>=FIST_ERROR)
@@ -381,7 +390,7 @@ job::eJobResult job::SendData(int confd)
 			}
 			
 			dbgline;
-			if(fistate==FIST_COMPLETE || (m_nSendPos < nNewSize && fistate>=FIST_DLGOTHEAD))
+			if(fistate==FIST_COMPLETE || (m_nSendPos < nGoodSize && fistate>=FIST_DLGOTHEAD))
 				break;
 			
 			dbgline;
@@ -423,7 +432,7 @@ job::eJobResult job::SendData(int confd)
 					//h.del("Content-Length"); // TESTCASE: Force chunked mode
 					
 					m_sPrependBuf=m_RespHead.frontLine+"\r\n";
-					
+
 					bool bSendBody(true);
 					// no data for error heads
 					if(m_RespHead.getStatus() != 200)
@@ -432,7 +441,7 @@ job::eJobResult job::SendData(int confd)
 					// make sure there is no junk inside
 					// if missing completely, chunked mode is to be used below
 					if(m_RespHead.h[header::CONTENT_LENGTH])
-					{ 
+					{
 						if(0==atol(m_RespHead.h[header::CONTENT_LENGTH]))
 							bSendBody=false;
 						
@@ -449,7 +458,7 @@ job::eJobResult job::SendData(int confd)
 					else
 					{
 						
-						if(! m_RespHead.h[header::CONTENT_LENGTH])
+						if( ! m_RespHead.h[header::CONTENT_LENGTH])
 						{
 							// unknown length but must have data, will have to improvise: prepare chunked transfer
 							if(m_bIsHttp11 != 0)
@@ -458,20 +467,30 @@ job::eJobResult job::SendData(int confd)
 							m_RespHead.set(header::TRANSFER_ENCODING, "chunked");
 						}
 						else
-						{ // has content length, can do optimizations?
+						{
+							// OK, has content length. Can do optimizations?
+							// Handle If-Modified-Since and Range headers
+
+							// we deal with them equally but need to know which to use
 							const char *pIfmo = m_pReqHead->h[header::RANGE] ? 
 									m_pReqHead->h[header::IFRANGE] : m_pReqHead->h[header::IF_MODIFIED_SINCE];
+
 							bool bGotLen(false);
 
-							bool bFreshnessForced=(m_type!=rechecks::FILE_INDEX || m_pReqHead->h[header::XORIG]);
+							// consider contents "fresh" for non-volatile data in "special cases"
+							bool bFreshnessForced=(m_type != rechecks::FILE_INDEX
+									|| m_pReqHead->h[header::ACNGFSMARK]);
+
 							// this plays well with lighttpd <-> Apt, but may not work with the alternative time format
 							// TODO: add format tolerant time comparer
-							bool bFreshnessTested=(pIfmo && m_RespHead.h[header::LAST_MODIFIED] &&
+							bool bIfModSeenAndChecked=(pIfmo && m_RespHead.h[header::LAST_MODIFIED] &&
 									0==strcmp(pIfmo, m_RespHead.h[header::LAST_MODIFIED]));
 							
 							// is it fresh? or is this relevant? or is range mode forced?
-							if(  bFreshnessForced || bFreshnessTested)		                          
+							if(  bFreshnessForced || bIfModSeenAndChecked)		                          
 							{
+								long nContLen=atol(m_RespHead.h[header::CONTENT_LENGTH]);
+
 								/*
 								 * Range: bytes=453291-
 								 * ...
@@ -480,51 +499,70 @@ job::eJobResult job::SendData(int confd)
 								 */
 								
 								const char *pRange=m_pReqHead->h[header::RANGE];
-								// work around BROKEN curl request
+
+								// working around a bug in old curl versions
 								if(!pRange)
 									pRange=m_pReqHead->h[header::CONTENT_RANGE];
+
 								if(pRange)
 								{
-									// extra check to avoid timeouts since the server
-									// may need time to get there. Don't care if the user is acngfs...
-									if(fistate==FIST_COMPLETE || m_pReqHead->h[header::XORIG])
+									off_t nRangeFrom(0), nRangeTo(0);
+									int nRangeItems=sscanf(pRange, "bytes=%ld-%ld",
+											&nRangeFrom, &nRangeTo);
+									// working around bad (old curl style) requests
+									if(nRangeItems <= 0)
+										nRangeItems=sscanf(pRange, "bytes %ld-%ld",
+												&nRangeFrom, &nRangeTo);
+
+									if(nRangeItems == 1) // open-end? set the limit
+										nRangeTo=nContLen-1;
+
+									/*
+									 * make sure that our client doesn't just
+									 * hang here while the download thread is
+									 * fetching from 0 to start position for
+									 * many minutes
+									 */
+
+									if(nRangeItems>0
+										&&
+										(  fistate==FIST_COMPLETE
+											// or can start sending within this range
+											|| (
+													fistate >= FIST_DLGOTHEAD
+													&& fistate <= FIST_COMPLETE
+													&& nGoodSize>=nRangeFrom
+												)
+											// don't care, found special hint from acngfs (kludge...)
+											|| m_pReqHead->h[header::ACNGFSMARK]
+											)
+										)
 									{
-										int r=sscanf(pRange, "bytes=%ld-%ld", 
-												&m_nSendPos, &m_nRangeLast);
-										// work around BROKEN curl request
-										if(r<=0)
-											r=sscanf(pRange, "bytes %ld-%ld", 
-												&m_nSendPos, &m_nRangeLast);
-										
-										if(r>0)
-										{
-											long nSize=atol(m_RespHead.h[header::CONTENT_LENGTH]);
-											
-											if(r==1) // not open-end, set the limit
-												m_nRangeLast=nSize-1;
-											
-											if(m_nSendPos>=nSize || m_nRangeLast>=nSize)
-												THROW_ERROR("416 Requested Range Not Satisfiable")
-											
-											m_sPrependBuf="HTTP/1.1 206 Partial Response\r\n";
-											char buf[100];
-											sprintf(buf, "Content-Length: %ld\r\n"
-													"Content-Range: bytes %ld-%ld/%ld\r\n",
-													m_nRangeLast-m_nSendPos+1,
-													m_nSendPos,
-													m_nRangeLast,
-													nSize
-													);
-											m_sPrependBuf+=buf;
-											bGotLen=true;
-										}
+										// detect errors, out-of-range case
+										if(nRangeTo>=nContLen || nRangeFrom>=nContLen || nRangeTo<nRangeFrom)
+											THROW_ERROR("416 Requested Range Not Satisfiable")
+
+										m_nSendPos = nRangeFrom;
+										m_nRangeLast = nRangeTo;
+										m_sPrependBuf="HTTP/1.1 206 Partial Response\r\nContent-Length: ";
+										char buf[200];
+										sprintf(buf, "%ld\r\nContent-Range: bytes %ld-%ld/%ld\r\n",
+												m_nRangeLast-m_nSendPos+1,
+												m_nSendPos,
+												m_nRangeLast,
+												nContLen
+												);
+										m_sPrependBuf+=buf;
+										bGotLen=true;
 									}
 								}
-								else if(!bFreshnessForced) 
-									// no range request found, and freshness test was ok -> report 304
+								else if(bIfModSeenAndChecked)
+								{
+									// file is fresh, and user sent if-mod-since -> fine
 									THROW_ERROR("304 Not Modified");
+								}
 							}
-							// has cont.len but header was not set in the range stuff above
+							// has cont-len available but this header was not set yet in the code above
 							if( !bGotLen)
 							{
 								m_sPrependBuf+=string("Content-Length: ")
@@ -537,14 +575,29 @@ job::eJobResult job::SendData(int confd)
 							+m_RespHead.h[header::LAST_MODIFIED]+"\r\n";
 					}
 					m_sPrependBuf+=header::GetInfoHeaders();
+
 					if(bSendBody)
 						m_sPrependBuf+="Content-Type: application/octet-stream\r\n";
+					else if(m_RespHead.h[header::LOCATION])
+					{
+							m_sPrependBuf+="Location: ";
+							m_sPrependBuf+=m_RespHead.h[header::LOCATION];
+							m_sPrependBuf+="\r\n";
+					}
+
 					if(m_RespHead.h[header::XORIG])
 						m_sPrependBuf+=string("X-Original-Source: ")+m_RespHead.h[header::XORIG]+"\r\n";
-										
-					m_sPrependBuf+=string("Connection: ")
-					+(m_bCloseCon?"close\r\n\r\n":"Keep-Alive\r\n\r\n");
 					
+					// only respond if the client has set it explicitely
+					if(m_pReqHead->h[header::CONNECTION])
+					{
+						// whatever the client wants
+						m_sPrependBuf+="Connection: ";
+						m_sPrependBuf+=m_pReqHead->h[header::CONNECTION];
+						m_sPrependBuf+="\r\n";
+					}
+						
+					m_sPrependBuf+="\r\n";
 					
 					if(m_pReqHead->type==header::HEAD)
 						m_backstate=STATE_ALLDONE; // simulated head is prepared but don't send stuff
@@ -579,27 +632,28 @@ job::eJobResult job::SendData(int confd)
 				}
 				case(STATE_SEND_PLAIN_DATA):
 				{
-					ldbg("STATE_SEND_PLAIN_DATA untill " << nNewSize);
+					ldbg("STATE_SEND_PLAIN_DATA untill " << nGoodSize);
 
 					// eof?
+#define GOTOENDE { m_state=STATE_FINISHJOB ; continue; }
 					if(m_nSendPos==m_pItem->m_nSizeChecked && fistate>=FIST_COMPLETE)
-						return m_bCloseCon ? R_DISCON : R_DONE;
+						GOTOENDE;
 					
-					if(nNewSize>m_nRangeLast)
-						nNewSize=m_nRangeLast+1;
+					if(nGoodSize>m_nRangeLast)
+						nGoodSize=m_nRangeLast+1;
 							
-					ldbg("~sendfile: on "<< m_nSendPos << " up to : " << nNewSize-m_nSendPos );
-					int n=sendfile_linux(confd, m_filefd, &m_nSendPos, nNewSize-m_nSendPos);
+					ldbg("~sendfile: on "<< m_nSendPos << " up to : " << nGoodSize-m_nSendPos );
+					int n=sendfile_linuxstyle(confd, m_filefd, &m_nSendPos, nGoodSize-m_nSendPos);
 					ldbg("~sendfile: " << n << " new m_nSendPos: " << m_nSendPos);
 
 					if(n>0)
 						m_nAllDataCount+=n;
 					
 					 // sendfile was not done? Store that size to skip fitem polling next time, or reset it
-					m_nCachedMaxSize = (nNewSize>m_nSendPos) ? nNewSize : 0;
+					m_nCachedMaxSize = (nGoodSize>m_nSendPos) ? nGoodSize : 0;
 					
 					if(m_nSendPos>m_nRangeLast)
-						return m_bCloseCon ? R_DISCON : R_DONE;
+						GOTOENDE;
 					
 					if(n<0)
 						THROW_ERROR("400 Client error");
@@ -610,7 +664,7 @@ job::eJobResult job::SendData(int confd)
 				case(STATE_SEND_CHUNK_HEADER):
 				{
 
-					m_nChunkRemainingBytes=nNewSize-m_nSendPos;
+					m_nChunkRemainingBytes=nGoodSize-m_nSendPos;
 
 					ldbg("STATE_SEND_CHUNK_HEADER for " << m_nChunkRemainingBytes);
 					// if not on EOF then the chunk must have remaining size (otherwise the state would have been changed)
@@ -632,9 +686,9 @@ job::eJobResult job::SendData(int confd)
 					ldbg("STATE_SEND_CHUNK_DATA");
 
 					if(m_nChunkRemainingBytes==0)
-						return m_bCloseCon ? R_DISCON : R_DONE; // yeah...
+						GOTOENDE; // yeah...
 
-					int n=sendfile_linux(confd, m_filefd, &m_nSendPos, m_nChunkRemainingBytes);
+					int n=sendfile_linuxstyle(confd, m_filefd, &m_nSendPos, m_nChunkRemainingBytes);
 					if(n<0)
 						THROW_ERROR("400 Client error");
 					m_nChunkRemainingBytes-=n;
@@ -650,14 +704,14 @@ job::eJobResult job::SendData(int confd)
 				}
 				case(STATE_SEND_BUFFER):
 				{
-					// special DELICATE state. Something fails here only when the connection is dead
-					// and this state will be used by the error handler to send feedback.
-					// ... or we send the report page
+          // Sends data in the local buffer, used for page or chunk headers
+          // sending, also error header sending. -> DELICATE STATE, should not
+          // be interrupted by exception or throw to another flow uncontrolled.
 					MYTRY
 					{
 						ldbg("prebuf sende: "<< m_sPrependBuf);
 						int r=send(confd, m_sPrependBuf.data(), m_sPrependBuf.length(), 
-								(m_backstate == STATE_ERRORCONT || m_backstate == STATE_NOWAYOUT) ? 0 : MSG_MORE);
+								(m_backstate == STATE_ERRORCONT || m_backstate == STATE_TODISCON) ? 0 : MSG_MORE);
 						if (r<0)
 						{
 							if (errno==EAGAIN || errno==EINTR || errno == ENOBUFS)
@@ -690,20 +744,28 @@ job::eJobResult job::SendData(int confd)
 				case (STATE_FAILMODE):
 					goto THROWN_ERROR;
 				
+				case(STATE_ALLDONE):
 				case (STATE_ERRORCONT):
-					if(m_bCloseCon)
+				case(STATE_FINISHJOB):
+					if(m_pReqHead && m_pReqHead->h[header::CONNECTION])
 					{
-						USRDBG(REPLEVEL, "Job done, Connection:close found or failed -> trigger connection shutdown");
+						// caller has set it properly, respond as needed
+						if(0==strcasecmp(m_pReqHead->h[header::CONNECTION], "close"))
+							return R_DISCON;
+						return R_DONE;
+					}
+					else
+					{
+						// now magic begins, work around weird behaviour of apt's http. 
+						// Did the client add more requests in the original bunch?
+												
+						if(nAllJobCount>1)
+							return R_DONE;
 						return R_DISCON;
 					}
-					USRDBG(REPLEVEL, "Job done, connection will persist");
-					return R_DONE;
 				
-
-				case(STATE_ALLDONE):
-					return R_DONE;
 				
-				case(STATE_NOWAYOUT):
+				case(STATE_TODISCON):
 				default:
 					return R_DISCON;
 					
@@ -716,10 +778,19 @@ job::eJobResult job::SendData(int confd)
 			ldbg("Processing error: " << m_sErrorMsgBuffer);
 			if(0==m_nAllDataCount) // don't send errors when data header is underway
 			{
-				m_sPrependBuf=string("HTTP/1.1 ")+m_sErrorMsgBuffer
-				+ "\r\nConnection: " + (m_bCloseCon?"close\r\n":"Keep-Alive\r\n")
-				+ header::GetInfoHeaders();
-								
+				m_sPrependBuf=string("HTTP/1.1 ")+m_sErrorMsgBuffer+"\r\n";
+				
+				// only respond if the client has set it explicitely
+				if(m_pReqHead->h[header::CONNECTION])
+				{
+					// whatever the client wants
+					m_sPrependBuf+="Connection: ";
+					m_sPrependBuf+=m_pReqHead->h[header::CONNECTION];
+					m_sPrependBuf+="\r\n";
+				}
+
+				m_sPrependBuf+=header::GetInfoHeaders();
+						
 				if(m_RespHead.h[header::XORIG])
 				{
 					m_sPrependBuf+="X-Original-Source: ";
