@@ -88,7 +88,7 @@ bool pkgimport::ProcessRegular(const string &sPath, const struct stat &stinfo)
 		if( rechecks::FILE_INDEX == kind )
 		{
 			//cout << "Is ifile\n";
-			m_volatileFiles.push_back(sPath);
+			m_volatileFiles.insert(sPath);
 		}
 	}
 	else /* also try to import ifiles when matched, why not?! if (rechecks::FILE_PKG == kind) */
@@ -127,9 +127,9 @@ bool pkgimport::ProcessRegular(const string &sPath, const struct stat &stinfo)
 	return true;
 }
 
+// linking not possible? different filesystems?
 bool FileCopy(const string &from, const string to)
 {
-	// not possible? different filesystems?
 	acbuf buf;
 	buf.init(50000);
 	int in(-1), out(-1);
@@ -169,14 +169,15 @@ bool FileCopy(const string &from, const string to)
 
 	}
 
-	close(in);
-	close(out);
+	forceclose(in);
+	forceclose(out);
 	return true;
 
-	error_copying: if (in>=0)
-		close(in);
+	error_copying: 
+	if (in>=0)
+		forceclose(in);
 	if (out>=0)
-		close(out);
+		forceclose(out);
 	return false;
 }
 bool pkgimport::LinkOrCopy(const MYSTD::string &from, const MYSTD::string &to)
@@ -185,7 +186,7 @@ bool pkgimport::LinkOrCopy(const MYSTD::string &from, const MYSTD::string &to)
 	
 	//ldbg("Moving " <<from<< " to " << to);
 	
-	// hardlinks don't work with symlinks properly...
+	// hardlinks don't work with symlinks properly, create a similar symlink then
 	struct stat stinfo, toinfo;
 	
 	if(0==lstat(from.c_str(), &stinfo) && S_ISLNK(stinfo.st_mode))
@@ -302,7 +303,7 @@ void pkgimport::Action(const string &cmd)
 		cfh=fopen(SCACHEFILE.c_str(), "wb");
 	}
 	if(cfh)
-		fprintf(cfh, "FMT2\n");
+		fprintf(cfh, "FMT3\n");
 	
 	for(tImportMap::const_iterator it=m_importMap.begin();
 	it!=m_importMap.end();it++)
@@ -328,7 +329,7 @@ void pkgimport::Action(const string &cmd)
 	}
 	if(cfh)
 	{
-		fclose(cfh);
+		forceFclose(cfh);
 		cfh=NULL;
 	}
 }
@@ -379,7 +380,7 @@ void pkgimport::_HandlePkgEntry(const tRemoteFileInfo &entry)
 	const string &sFrom=hit->second.sPath;
 
 	SendMsg(string("<font color=green>HIT: ")+sFrom
-			+ "<br>DESTINATION: "+sDest+"</font><br>\n");
+			+ "<br>\nDESTINATION: "+sDest+"</font><br>\n");
 
 	// linking and moving would shred them when the link leads to the same target
 	struct stat tmp1, tmp2;
@@ -435,7 +436,7 @@ void pkgimport::_LoadKeyCache(const string & sFileName)
 {
 	filereader reader;
 	string sLine;
-	if(!reader.OpenFile(sFileName)||!reader.GetOneLine(sLine)||sLine!="FMT2")
+	if(!reader.OpenFile(sFileName)||!reader.GetOneLine(sLine)||sLine!="FMT3")
 		return;
 
 	/*typedef MYSTD::map<tFingerprint, tImpFileInfo, ltfingerprint> tImportMap;
