@@ -6,13 +6,15 @@
 
 #include "acfg.h" 
 
-rechecks::rechecks() {
+rechecks::rechecks() : m_nErrors(0) {
 	//printf("%s\n%s\n", a, b);
-    int rc;
-    rc=regcomp(&rePfiles, acfg::pfilepat.c_str(), REG_EXTENDED);
-    ASSERT(!rc);
+    
+    int rc = regcomp(&rePfiles, acfg::pfilepat.c_str(), REG_EXTENDED);
+    m_nErrors+=(rc!=0);
+        
     rc=regcomp(&reIfiles, acfg::vfilepat.c_str(), REG_EXTENDED);
-    ASSERT(!rc);
+    m_nErrors+=(rc!=0);
+        
     //ldbg("reIfiles: " << rc);
     //rc=regcomp(&rePoison, "[^a-zA-Z0-9\\-+_~./]", REG_EXTENDED);
     //rc=regcomp(&rePoison, "[^a-zA-Z0-9._~+/-]", REG_EXTENDED);
@@ -24,6 +26,7 @@ rechecks::~rechecks() {
     //regfree(&rePoison);
     regfree(&reIfiles);
     regfree(&rePfiles);
+    
 }
 
 rechecks::eFileKind rechecks::getFiletype(const char *in) {
@@ -37,6 +40,20 @@ rechecks::eFileKind rechecks::getFiletype(const char *in) {
     return FILE_INVALID;
 }
 
+bool rechecksEx::IsInWhitelist(const char *s)
+{
+	return (!regexec(&reWfiles, s, 0, NULL, 0));
+}
+
+rechecksEx::rechecksEx()
+{
+   	m_nErrors+=(0!=regcomp(&reWfiles, acfg::wfilepat.c_str(), REG_EXTENDED));    
+}
+rechecksEx::~rechecksEx()
+{
+	regfree(&reWfiles); 
+}
+
 /*
 int main(int argc, char **argv) {
     rechecks checka;
