@@ -32,6 +32,8 @@ void reportgen::Run(const string &s)
 	SendDecoration(false);
 }
 
+#define nRatWid 12
+
 void reportgen::GenReport(const string & sCmd)
 {
 	filereader reader;
@@ -44,7 +46,7 @@ void reportgen::GenReport(const string & sCmd)
 	next_line:
 	string sLine;
 	//double factor;
-	int nRatWid(200);
+	//int nRatWid(200);
 	vector<char> buf(4000);
 	
 	while(reader.GetOneLine(sLine))
@@ -87,9 +89,13 @@ void reportgen::GenReport(const string & sCmd)
 			*/
 			case ('W'):
 			{
-				nRatWid=atoi(sLine.c_str()+pos+1);
+				/* stupid bars disabled, define as constant now
+							
+							nRatWid=atoi(sLine.c_str()+pos+1);
+							*/
 				goto next_line;
 			}
+			
 			case('A'):
 			{
 				if(acfg::exfailabort)
@@ -100,9 +106,10 @@ void reportgen::GenReport(const string & sCmd)
 			{
 				if(!m_bVerbose)
 				{
-				SendMsg(string("<tr><td valign=top><form action=\"#top\">"
-						"<input type=\"submit\" name=\"doCount\" value=\"Count Data\"></form>")
-						+"</td><td colspan=2 valign=top><font size=-2><i>Not calculated, click \"Count data\"</i></font></td></tr>");
+				SendMsg("<tr><td class=\"colcont\"><form action=\"#top\">"
+						"<input type=\"submit\" name=\"doCount\" value=\"Count Data\"></form>"
+						"</td><td class=\"colcont\" colspan=8 valign=top><font size=-2>"
+						"<i>Not calculated, click \"Count data\"</i></font></td></tr>");
 				}
 				else
 				{
@@ -111,6 +118,13 @@ void reportgen::GenReport(const string & sCmd)
 					for (vector<aclog::tRowData>::iterator it=data.begin(); it
 							!=data.end(); it++)
 					{
+						
+						unsigned long reqMax = max(it->reqIn, it->reqOut);
+						uint64_t dataMax =  max(it->byteIn, it->byteOut);
+						
+						if( 0 == dataMax || 0 == reqMax)
+							continue;
+												
 						char tbuf[50];
 						size_t zlen(0);
 						ctime_r(&it->from, tbuf);
@@ -130,9 +144,34 @@ void reportgen::GenReport(const string & sCmd)
 									" - " TIMEFORMAT, tmp))
 								goto time_error;
 						}
+						snprintf(&buf[0], buf.size(), 
+								"<tr bgcolor=\"white\">"
 
-						snprintf(&buf[0], buf.size(), (sLine.substr(pos+1)+sCRLF).c_str(), tbuf, it->count
-								/1048576, int(it->ratioSent*nRatWid), int((1.0-it->ratioSent)*nRatWid));
+								"<td class=\"colcont\">%s</td>"
+
+								"<td class=\"coltitle\"><span>&nbsp;</span></td>"
+								
+								"<td class=\"colcont\">%lu (%2.2f%%)</td>"
+								"<td class=\"colcont\">%lu (%2.2f%%)</td>"
+								"<td class=\"colcont\">%lu</td>"
+
+								"<td class=\"coltitle\"><span>&nbsp;</span></td>"
+								
+								"<td class=\"colcont\">%2.2f MiB (%2.2f%%)</td>"
+								"<td class=\"colcont\">%2.2f MiB (%2.2f%%)</td>"
+								"<td class=\"colcont\">%2.2f MiB</td>"
+								"</tr>"
+								, 
+								tbuf,
+								reqMax - it->reqIn, double(reqMax - it->reqIn) / reqMax * 100, // hitcount
+								it->reqIn, double(it->reqIn) / reqMax * 100, // misscount
+								reqMax,
+								
+								double(dataMax - it->byteIn)/1048576, double(dataMax - it->byteIn)/dataMax*100, // hitdata
+								double(it->byteIn)/1048576, double(it->byteIn)/dataMax*100, // missdata
+								double(dataMax)/1048576
+								
+								); //, int(it->ratioSent*nRatWid), int((1.0-it->ratioSent)*nRatWid));
 						SendMsg(&buf[0]);
 						continue;
 						time_error: SendMsg(" Invalid time value detected, check the stats database. ");
