@@ -25,16 +25,6 @@ void showinfo::Action(const MYSTD::string & src)
 {
 }
 
-inline void StrSubst(string &contents, const string &from, const string &to)
-{
-	tStrPos pos;
-	while (stmiss!=(pos=contents.find(from)))
-	{
-		contents.replace(pos, from.length(), to);
-		pos+=to.length();
-	}
-}
-
 void showinfo::Run(const string &)
 {
 	SendDecoration(true, "404", NULL);
@@ -48,7 +38,7 @@ void showinfo::Run(const string &)
 	{
 		string contents(pagebuf.rptr(), pagebuf.size());
 		SetStyle(contents);
-		StrSubst(contents, "$SERVERIP", string("[")+GetHostname()+"]");
+		StrSubst(contents, "$SERVERIP", GetHostname());
 		StrSubst(contents, "$SERVERPORT", acfg::port.c_str());
 		StrSubst(contents, "$REPAGE", sPathSepUnix+acfg::reportpage);
 		
